/*
    YAM - Yet Another Mod player
    Copyright 2001, 2002 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "modfile.h"
#include "modplayer.h"
#include "audio.h"

void printInfo(ModFile *mf) {
	int i, n;
	printf("%s\n",mf->name);

	n = mf->numSamples/2;
	if (mf->numSamples%2==1) n++;

	for (i = 0; i < n; i++) {
		if (i+n<mf->numSamples) {
			printf("%2d %-22s     %2d %s\n",i+1,mf->samples[i].name,i+n+1,mf->samples[i+n].name);
		} else {
			printf("%2d %s\n",i+1,mf->samples[i].name);
		}
	}
}

void rowCallback(int row, int patternNr, int orderNr, int length) {
	char buffer[50];
	int l;
	sprintf(buffer,"%d",length);
	l = strlen(buffer);
	printf("\r[%2d] [%3d] [%*d/%d]",row+1,patternNr,l,orderNr+1,length);
	fflush(stdout);
}

int main(int argc, char *argv[]) {
	FILE *in;

	if (argc<2) {
		printf("%s <mod file>\n",argv[0]);
		return 0;
	}
	if ((in = fopen(argv[1],"rb"))==NULL) {
		perror(argv[1]);
		return 1;
	}
	ModFile *mf = new ModFile(in);
	fclose(in);

	printInfo(mf);

	if (!init_audio()) return 1;
	ModPlayer *mp = new ModPlayer(mf,channels,samplerate,bitdepth,sig,littleendian);
	mp->addRowCallback(rowCallback);
//	play_mod(mp);
	start_play(ModPlayer_read,mp);
	stop_audio();

	delete mf;
	delete mp;

	printf("\r");
	fflush(stdout);

	return 0;
}

