/*
    YAM - Yet Another Mod player
    Copyright 2001, 2002 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#include <stdio.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include "audio.h"

#define DEVICE "/dev/dsp"

static int audio;
static int stop;
static int fragment_size;
static int playing = 0;

int channels = 2;
int samplerate = 44100;
int bitdepth = 16;
int sig = 1;
int littleendian = 1;

int init_audio(void) {
	int arg;
	audio_buf_info info;
	int format;
	sig = 1;
	bitdepth = 16;
	littleendian = 1;

	if ((audio = open(DEVICE,O_WRONLY)) < 0) {
		perror(DEVICE);
		return 0;
	}

#define NUM_FRAGMENTS 16
#define FRAGMENT_SIZE 13
/* fragment_size = 2^FRAGMENT_SIZE */
	arg = NUM_FRAGMENTS<<16 | FRAGMENT_SIZE;
/*	ioctl(audio,SNDCTL_DSP_SETFRAGMENT,arg);
*/	ioctl(audio,SNDCTL_DSP_GETOSPACE,&info);
	fragment_size = info.fragsize;

	format = AFMT_S16_LE;
	if (ioctl(audio,SNDCTL_DSP_SETFMT,&format)<0) {
		perror("SNDCTL_DSP_SETFMT");
		return 0;
	}
	switch (format) {
		case AFMT_S16_LE:
			sig = 1;
			bitdepth = 16;
			littleendian = 1;
			break;
		case AFMT_S16_BE:
			sig = 1;
			bitdepth = 16;
			littleendian = 0;
			break;
		case AFMT_U16_LE:
			sig = 0;
			bitdepth = 16;
			littleendian = 1;
			break;
		case AFMT_U16_BE:
			sig = 0;
			bitdepth = 16;
			littleendian = 0;
			break;
		case AFMT_U8:
			sig = 0;
			bitdepth = 8;
			break;
		case AFMT_S8:
			sig = 1;
			bitdepth = 8;
			break;
		default:
			fprintf(stderr,"Unable to set audio format\n");
			return 0;
			break;
	}
	channels = 2;
	if (ioctl(audio,SNDCTL_DSP_CHANNELS,&channels)<0) {
		perror("SNDCTL_DSP_CHANNELS");
		return 0;
	}
	samplerate = 44100;
	if (ioctl(audio,SNDCTL_DSP_SPEED,&samplerate)<0) {
		perror("SNDCTL_DSP_SPEED");
		return 0;
	}

	return 1;
}

void stop_audio(void) {
	close(audio);
}

void start_play(read_func rf, void *ptr) {
	unsigned char *buffer = malloc(fragment_size);
	int n;
	stop = 0;
	playing = 1;
	while (!stop) {
		n = rf(ptr,buffer,fragment_size);
		if (n<=0) break;
		write(audio,buffer,n);
	}
	playing = 0;
	free(buffer);
}

int is_playing(void) {
	return playing;
}

void break_play(void) {
	stop = 1;
}

